# Copyright (c) 2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

import base64
import re
import urllib

from AccessControl.SecurityManagement import (
    getSecurityManager, setSecurityManager, noSecurityManager)
from transaction import commit

from infrae.testing import Zope2Layer, suite_from_package, TestCase
from infrae.wsgi.publisher import WSGIApplication
from wsgi_intercept.mechanize_intercept import Browser as BaseInterceptBrowser
from zope.testbrowser.browser import Browser as ZopeTestBrowser
from zope.site.hooks import getSite, setSite, setHooks
import wsgi_intercept

# List of hostname where the test browser/http function replies to
TEST_HOSTS = ['localhost', '127.0.0.1']


class InterceptBrowser(BaseInterceptBrowser):

    default_schemes = ['http']
    default_others = ['_http_error', '_http_default_error']
    default_features = ['_redirect', '_cookies', '_referer', '_refresh',
                        '_equiv', '_basicauth', '_digestauth']


class Browser(ZopeTestBrowser):
    """Override the zope.testbrowser.browser.Browser interface so that it
    uses PatchedMechanizeBrowser
    """

    def __init__(self, *args, **kwargs):
        kwargs['mech_browser'] = InterceptBrowser()
        ZopeTestBrowser.__init__(self, *args, **kwargs)

    @property
    def status(self):
        return '%d %s' % (
            self.mech_browser._response.code,
            self.mech_browser._response.msg)


# Compatibility helpers to behave like zope.app.testing

basicre = re.compile('Basic (.+)?:(.+)?$')
def auth_header(header):
    """This function takes an authorization HTTP header and encode the
    couple user, password into base 64 like the HTTP protocol wants
    it.
    """
    match = basicre.match(header)
    if match:
        u, p = match.group(1, 2)
        if u is None:
            u = ''
        if p is None:
            p = ''
        auth = base64.encodestring('%s:%s' % (u, p))
        return 'Basic %s' % auth[:-1]
    return header


def is_wanted_header(header):
    """Return True if the given HTTP header key is unwanted.
    """
    key, value = header
    return key.lower() not in ('x-content-type-warning', 'x-powered-by')


class TestBrowserWSGIResult(object):
    """Call a WSGI Application and return its result.

    Backup the ZCA local site before calling the wrapped application and
    restore it when done.
    """

    def __init__(self, app, connection, environ, start_response):
        self.app = app
        self.connection = connection
        self.environ = environ
        self.start_response = start_response
        self.__result = None
        self.__next = None
        self.__site = None
        self.__security = None

    def __iter__(self):
        return self

    def next(self):
        if self.__next is None:
            # Backup ZCA site and security manager
            self.__site = getSite()
            self.__security = getSecurityManager()
            noSecurityManager()
            self.__result = self.app(self.environ, self.start_response)
            self.__next = iter(self.__result).next
        return self.__next()

    def close(self):
        if self.__result is not None:
            if hasattr(self.__result, 'close'):
                self.__result.close()
        if self.__site is not None:
            setSite(self.__site)
            setHooks()
        if self.__security is not None:
            setSecurityManager(self.__security)
        self.connection.sync()


class TestBrowserMiddleware(object):
    """This middleware makes the WSGI application compatible with the
    HTTPCaller behavior defined in zope.app.testing.functional:
    - It honors the X-zope-handle-errors header in order to support
      zope.testbrowser Browser handleErrors flag.
    - It modifies the HTTP Authorization header to encode user and
      password into base 64 if it is Basic authentication.
    """

    def __init__(self, app, connection, handle_errors):
        assert isinstance(handle_errors, bool)
        self.app = app
        self.default_handle_errors = str(handle_errors)
        self.connection = connection

    def __call__(self, environ, start_response):
        # Handle debug mode
        if 'wsgi.handleErrors' not in environ:
            handle_errors = environ.get(
                'HTTP_X_ZOPE_HANDLE_ERRORS', self.default_handle_errors)
            environ['wsgi.handleErrors'] = handle_errors == 'True'

        # Handle authorization
        auth_key = 'HTTP_AUTHORIZATION'
        if environ.has_key(auth_key):
            environ[auth_key] = auth_header(environ[auth_key])

        # Remove unwanted headers
        def application_start_response(status, headers):
            headers = filter(is_wanted_header, headers)
            return start_response(status, headers)

        commit()
        return TestBrowserWSGIResult(
            self.app, self.connection, environ, application_start_response)


class BrowserLayer(Zope2Layer):
    """Functional test layer.
    """

    def _create_wsgi_application(self):
        # Create Zope WSGI Application. You can change this method if
        # you whish to change the tested WSGI application.
        return WSGIApplication(
            self._application, self._transaction_manager)

    def testSetUp(self):
        super(BrowserLayer, self).testSetUp()
        wsgi_app = self._create_wsgi_application()

        def factory(handle_errors=True):
            return TestBrowserMiddleware(
                wsgi_app, self._test_connection, handle_errors)
        self._test_wsgi_application = TestBrowserMiddleware(
            wsgi_app, self._test_connection, True)

        for host in TEST_HOSTS:
            wsgi_intercept.add_wsgi_intercept(host, 80, factory)

    def testTearDown(self):
        for host in TEST_HOSTS:
            wsgi_intercept.remove_wsgi_intercept(host, 80)
        self._test_wsgi_application = None
        super(BrowserLayer, self).testTearDown()


class ResponseParser(object):
    """This behave like a Response object returned by HTTPCaller of
    zope.app.testing.functional.
    """

    def __init__(self, data):
        self.data = data

    def getStatus(self):
        line = self.getStatusString()
        status, rest = line.split(' ', 1)
        return int(status)

    def getStatusString(self):
        status_line = self.data.split('\n', 1)[0]
        protocol, status_string = status_line.split(' ', 1)
        return status_string

    def getHeader(self, name, default=None):
        return self.getHeaders().get(name, default)

    def getHeaders(self):
        without_body = self.data.split('\n\n', 1)[0]
        headers_text = without_body.split('\n', 1)[1]
        headers = {}
        for header in headers_text.split('\n'):
            header_name, header_value = header.split(':', 1)
            header_value = header_value.strip()
            if header_name in headers:
                if isinstance(headers[header_name], list):
                    headers[header_name].append(header_value)
                else:
                    headers[header_name] = [headers[header_name], header_value]
            else:
                headers[header_name] = header_value
        return headers

    def getBody(self):
        parts = self.data.split('\n\n', 1)
        if len(parts) < 2:
            return ''
        return parts[1]

    def getOutput(self):
        return self.data

    __str__ = getOutput


def http(string, handle_errors=False, headers=None, parsed=False,
         data=None, auth=""):
    """This function behave like the HTTPCaller of
    zope.app.testing.functional.
    """
    string += "\r\n"
    body = ""
    if headers is None:
        headers = {}
    if auth:
        headers['Authorization'] = 'Basic %s' % (
            "%s:%s" % (auth, auth,)).encode('base64')
    if data is not None:
        body = urllib.urlencode(data)
        headers['Content-Type'] = 'application/x-www-form-urlencoded'
        headers['Content-Length'] = len(body)

    for key, value in headers.items():
        string += '%s: %s\r\n' % (key, value)

    if body:
        string += body

    key = ('localhost', 80)

    if key not in wsgi_intercept._wsgi_intercept:
        raise ValueError("Test not runned in the proper layer.")

    (app_fn, script_name) = wsgi_intercept._wsgi_intercept[key]
    app = app_fn(handle_errors=handle_errors)

    socket = wsgi_intercept.wsgi_fake_socket(app, 'localhost', 80, '')
    socket.sendall(string)
    result = socket.makefile()
    response = result.getvalue()
    if parsed:
        return ResponseParser(response)
    return response



# Copyright (c) 2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from cStringIO import StringIO
import unittest

from infrae.testing import ZCMLLayer
from infrae.wsgi.publisher import WSGIRequest, set_virtual_host
from infrae.wsgi.tests.mockers import MockApplication
import infrae.wsgi

TEST_REQUEST="GET /root HTTP/1.1\r\nHost:infrae.com\r\n\r\n"
TEST_ENVIRON = {'SERVER_NAME': 'infrae.com', 'SERVER_PORT': '80'}


class RequestTestCase(unittest.TestCase):
    """Test the WSGI request Virtual Host support.
    """
    layer = ZCMLLayer(infrae.wsgi)

    def setUp(self):
        self.request = WSGIRequest(StringIO(TEST_REQUEST), TEST_ENVIRON, None)
        self.request['PARENTS'] = [MockApplication(),]

    def test_simple(self):
        self.assertEqual(
            self.request.physicalPathToURL('/root'), 'http://infrae.com/root')
        self.assertEqual(
            self.request.getURL(), 'http://infrae.com')

    def test_virtual_host(self):
        set_virtual_host(self.request, 'https://silva.net')

        self.assertEqual(
            self.request.physicalPathToURL('/root'), 'https://silva.net/root')
        self.assertEqual(
            self.request.getURL(), 'https://silva.net')

    def test_virtual_host_and_path(self):
        set_virtual_host(self.request, 'http://zope.org/products/silva')

        self.assertEqual(
            self.request.physicalPathToURL('/root'),
            'http://zope.org/products/silva/root')
        self.assertEqual(
            self.request.getURL(),
            'http://zope.org/products/silva')


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(RequestTestCase))
    return suite



# Copyright (c) 2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

import unittest

import infrae.wsgi
from infrae.wsgi.testing import BrowserLayer, Browser, http


class MockWSGIApplication(object):
    """This mock WSGI application let us test the Browser/http method
    integration.
    """

    def __init__(self):
        self.__environ = None
        self.__result_status = '200 OK'
        self.__result_headers = ()

    def get_environ(self):
        return self.__environ

    def __call__(self, environ, start_response):
        self.__environ = environ
        write = start_response(self.__result_status, self.__result_headers)
        return ["Test succeed"]


class BrowserTestLayer(BrowserLayer):
    """A BrowserTestLayer where the Zope WSGI application is replaced
    with the mock WSGI application.
    """

    def testSetUp(self):
        self.test_wsgi_application = MockWSGIApplication()
        super(BrowserTestLayer, self).testSetUp()

    def testTearDown(self):
        super(BrowserTestLayer, self).testTearDown()
        self.test_wsgi_application = None

    def _create_wsgi_application(self):
        return self.test_wsgi_application


FunctionalLayer = BrowserTestLayer(infrae.wsgi)


class BrowserTestCase(unittest.TestCase):
    """Test the test browser.
    """
    layer = FunctionalLayer

    def setUp(self):
        self.wsgi_application = self.layer.test_wsgi_application

    def test_open(self):
        """Just test to access a URL.
        """
        browser = Browser()
        browser.open('http://localhost/index.html')
        environ = self.wsgi_application.get_environ()

        self.assertEqual(environ['wsgi.handleErrors'], True)
        self.assertEqual(environ['PATH_INFO'], '/index.html')
        self.assertEqual(environ['REQUEST_METHOD'], 'GET')

        self.assertEqual(browser.status, '200 OK')
        self.assertEqual(browser.contents, 'Test succeed')

    def test_handle_errors(self):
        """Test that the flag handleError on the browser switch the
        wsgi debug mode.
        """
        browser = Browser()
        browser.handleErrors = False
        browser.open('http://localhost/index.html')
        environ = self.wsgi_application.get_environ()

        self.assertEqual(environ['wsgi.handleErrors'], False)
        self.assertEqual(environ['PATH_INFO'], '/index.html')
        self.assertEqual(environ['REQUEST_METHOD'], 'GET')

    def test_authenticate(self):
        """Test addHeader/authentication header.
        """
        browser = Browser()
        browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        browser.open('http://localhost/index.html')
        environ = self.wsgi_application.get_environ()

        self.assertEqual(environ['HTTP_AUTHORIZATION'], 'Basic bWdyOm1ncnB3')

    def test_authenticate_base64(self):
        """Test addHeader/authentication header with an already
        encoded header.
        """
        browser = Browser()
        browser.addHeader('Authorization', 'Basic bWdyOm1ncnB3')
        browser.open('http://localhost/index.html')
        environ = self.wsgi_application.get_environ()

        self.assertEqual(environ['HTTP_AUTHORIZATION'], 'Basic bWdyOm1ncnB3')


class HTTPTestCase(unittest.TestCase):
    """Test the test http function.
    """
    layer = FunctionalLayer

    def setUp(self):
        self.wsgi_application = self.layer.test_wsgi_application

    def test_open(self):
        """Test simple open.
        """
        result = http('GET /index.html HTTP/1.0')
        environ = self.wsgi_application.get_environ()

        self.assertEqual(result, 'HTTP/1.0 200 OK\n\nTest succeed')
        self.assertEqual(environ['wsgi.handleErrors'], False)
        self.assertEqual(environ['PATH_INFO'], '/index.html')
        self.assertEqual(environ['REQUEST_METHOD'], 'GET')

    def test_handle_error(self):
        """Test the handle error flag.
        """
        result = http('POST /index.html HTTP/1.0', handle_errors=True)
        environ = self.wsgi_application.get_environ()

        self.assertEqual(result, 'HTTP/1.0 200 OK\n\nTest succeed')
        self.assertEqual(environ['wsgi.handleErrors'], True)
        self.assertEqual(environ['PATH_INFO'], '/index.html')
        self.assertEqual(environ['REQUEST_METHOD'], 'POST')

    def test_open_parsed(self):
        """Test parsing the result.
        """
        result = http('GET /index.html HTTP/1.0', parsed=True)
        environ = self.wsgi_application.get_environ()

        self.assertEqual(result.getStatus(), 200)
        self.assertEqual(result.getStatusString(), '200 OK')
        self.assertEqual(result.getBody(), 'Test succeed')
        self.assertEqual(result.getOutput(), 'HTTP/1.0 200 OK\n\nTest succeed')
        self.assertEqual(str(result), 'HTTP/1.0 200 OK\n\nTest succeed')
        self.assertEqual(environ['wsgi.handleErrors'], False)
        self.assertEqual(environ['PATH_INFO'], '/index.html')
        self.assertEqual(environ['REQUEST_METHOD'], 'GET')


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(BrowserTestCase))
    suite.addTest(unittest.makeSuite(HTTPTestCase))
    return suite




# Copyright (c) 2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

import unittest

import infrae.wsgi
from infrae.wsgi.testing import BrowserLayer, Browser


class FunctionalTestCase(unittest.TestCase):
    """Functional testing.
    """
    layer = BrowserLayer(infrae.wsgi)

    def setUp(self):
        self.browser = Browser()
        self.browser.handleErrors = True
        self.browser.raiseHttpErrors = False

    def test_default_view(self):
        self.browser.open('http://localhost')
        self.assertEqual(self.browser.title, 'Zope QuickStart')
        self.assertEqual(self.browser.status, '200 OK')

    def test_notfound(self):
        self.browser.open('http://localhost/nowhere')
        self.assertEqual(self.browser.status, '404 Not Found')


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(FunctionalTestCase))
    return suite



# Copyright (c) 2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

import threading
import logging
import pdb
import sys

from Zope2 import startup
from infrae.wsgi.publisher import WSGIApplication
from zope.event import notify
from zope.processlifetime import ProcessStarting
import App.config
import Zope2

logger = logging.getLogger('infrae.wsgi')
bootstrap_lock = threading.Lock()
bootstrap_done = False


def configure_zope(config_filename, debug_mode=False):
    """Read zope.conf with zdaemon^Wzcrap.
    """
    from Zope2.Startup import options, handlers
    import AccessControl

    del sys.argv[1:]
    opts = options.ZopeOptions()
    opts.configfile = config_filename
    opts.realize(raise_getopt_errs=0)

    handlers.handleConfig(opts.configroot, opts.confighandlers)
    AccessControl.setImplementation(
        opts.configroot.security_policy_implementation)
    AccessControl.setDefaultBehaviors(
        not opts.configroot.skip_ownership_checking,
        not opts.configroot.skip_authentication_checking,
        opts.configroot.verbose_security)
    App.config.setConfiguration(opts.configroot)
    set_zope_debug_mode(debug_mode)


def set_zope_debug_mode(debug_mode):
    """Set the Zope debug mode to the given value.
    """
    config = App.config.getConfiguration()
    config.debug_mode = debug_mode and 1 or 0
    import Globals
    Globals.DevelopmentMode = config.debug_mode


def mount_all_databases():
    """Call this to mount all available Zope databases.
    """
    config = App.config.getConfiguration()
    connection = Zope2.DB.open()
    root = connection.root()['Application']
    for path, name in config.dbtab.listMountPaths():
        if path != '/':
            logger.info("Mount %s on %s" % (name, path))
            root.unrestrictedTraverse(path)
    connection.close()


def boot_zope(config_filename, debug_mode=False):
    """Boot Zope.
    """
    global bootstrap_done
    bootstrap_lock.acquire()
    try:
        if bootstrap_done:
            logger.info("Zope already configured, skipping configuration")
            return
        configure_zope(config_filename, debug_mode)
        logger.info("Zope configured")

        try:
            startup()
            mount_all_databases()
        except Exception:
            if debug_mode:
                # If debug_mode is on, debug possible starting errors.
                print "%s:" % sys.exc_info()[0]
                print sys.exc_info()[1]
                pdb.post_mortem(sys.exc_info()[2])
            raise

        # Some products / Zope code reset the debug mode. Re-set it again.
        set_zope_debug_mode(debug_mode)

        # Notify start of application
        notify(ProcessStarting())
        bootstrap_done = True
        logger.info("Zope started")
    finally:
        bootstrap_lock.release()


def zope2_application_factory(global_conf, zope_conf, **options):
    """Build a Zope2 WSGI application.
    """
    debug_mode = options.get('debug_mode', 'off') == 'on'
    boot_zope(zope_conf, debug_mode)

    return WSGIApplication(
        Zope2.bobo_application,
        Zope2.zpublisher_transactions_manager,
        not debug_mode)



# Copyright (c) 2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from urlparse import urlparse
from tempfile import TemporaryFile
from cStringIO import StringIO
import socket
import types

from AccessControl.SecurityManagement import noSecurityManager
from Acquisition.interfaces import IAcquirer
from ZPublisher.Publish import Retry
from ZPublisher.Request import Request
from ZPublisher.mapply import mapply
from ZPublisher.pubevents import PubStart, PubSuccess, PubFailure, \
    PubBeforeCommit, PubAfterTraversal, PubBeforeAbort
from ZODB.POSException import ConflictError
from zope.component import queryMultiAdapter
from zope.event import notify
from zope.interface import implements
from zope.site.hooks import getSite
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.security.management import newInteraction, endInteraction
import Zope2
import zExceptions

from infrae.wsgi.errors import DefaultError
from infrae.wsgi.response import WSGIResponse, AbortPublication
from infrae.wsgi.log import logger, log_last_error, ErrorSupplement

CHUNK_SIZE = 1<<16              # 64K

def set_virtual_host(request, virtual_host):
    url = urlparse(virtual_host)
    if ':' in url.netloc:
        hostname, port = url.netloc.split(':', 1)
        request.setServerURL(url.scheme, hostname, int(port))
    else:
        request.setServerURL(url.scheme, url.netloc)
    request.setVirtualRoot(url.path.split('/'))


def call_object(obj, args, request):
    return apply(obj, args)


def missing_name(name, request):
    if name == 'self':
        return request['PARENTS'][0]
    raise zExceptions.BadRequest(name)


def dont_publish_class(klass, request):
    raise zExceptions.Forbidden("class %s" % klass.__name__)


class WSGIRequest(Request):
    """A WSGIRequest have a default skin.
    """
    implements(IDefaultBrowserLayer)


class WSGIResult(object):
    """Iterator to wrap Zope result, in order to commit/abort the
    transaction at the end of the iterator iteration, and to close the
    request at the end.
    """

    def __init__(self, request, publisher, data):
        self.__next = iter(data).next
        self.request = request
        self.publisher = publisher
        self.__iteration_done = False

    def next(self):
        try:
            return self.__next()
        except StopIteration:
            self.__iteration_done = True
            raise

    def __iter__(self):
        return self

    def close(self):
        try:
            if not self.__iteration_done:
                self.publisher.abort()
            else:
                self.publisher.finish()
        except ConflictError:
            self.publisher.abort()
            raise
        finally:
            # Always close the request, even if there errors while committing.
            self.request.close()


class WSGIPublication(object):
    """Publish a request through WSGI.
    """

    def __init__(self, app, request, response):
        self.app = app
        self.request = request
        self.response = response
        self.data_sent = False
        self.publication_done = False

    def __safe_callback(self, func, *args, **kwargs):
        try:
            func(*args, **kwargs)
        except Exception:
            log_last_error(self.request, self.response)

    def start(self):
        """Start the publication process.
        """
        notify(PubStart(self.request))
        newInteraction()
        noSecurityManager()
        self.app.transaction.begin()
        self.request.processInputs()

    def commit(self):
        """Commit results of the publication.
        """
        self.__safe_callback(notify, PubBeforeCommit(self.request))
        self.app.transaction.commit()
        endInteraction()
        self.__safe_callback(notify, PubSuccess(self.request))

    def abort(self):
        """Abort the current publication process.
        """
        self.__safe_callback(notify, PubBeforeAbort(self.request, None, False))
        self.app.transaction.abort()
        endInteraction()
        self.__safe_callback(notify, PubFailure(self.request, None, False))

    def finish(self):
        """End the publication process, by either committing the
        transaction or aborting it.
        """
        if self.publication_done:
            return
        if self.response.getStatus() < 404:
            # We want to commit the transaction in case of redirects
            # and unauthorized.
            self.commit()
        else:
            self.abort()
        self.publication_done = True

    def result(self):
        """Return the result of the response, and commit if we don't
        plan to stream data/or had stream data.
        """
        self.data_sent, data = self.response.getWSGIResponse()
        if not self.data_sent:
            # If we didn't send any data yet, commit now: if the commit
            # fail, we can retry the request since we didn't send
            # anything.
            self.finish()
        return data

    def error(self, error, last_known_obj):
        """Render and log an error.
        """
        if not IAcquirer.providedBy(last_known_obj):
            last_known_site = getSite()
            if last_known_site is not None:
                last_known_obj = last_known_site
        context = DefaultError(error)
        if IAcquirer.providedBy(last_known_obj):
            context = context.__of__(last_known_obj)
        error_page = queryMultiAdapter(
            (context, self.request), name='error.html')

        if error_page is not None:
            try:
                error_result = error_page()
                if error_result is not None:
                    self.response.setBody(error_result)
            except Exception as error:
                log_last_error(
                    self.request, self.response, obj=last_known_obj,
                    extra=u"Error while rendering error message\n")
                self.response.setStatus(500)
                self.response.setBody(ERROR_WHILE_RENDERING_ERROR_TEMPLATE)
        else:
            logger.error('No action defined for last exception')
            self.response.setStatus(500)
            self.response.setBody(DEFAULT_ERROR_TEMPLATE)

    def publish(self):
        """Publish the request into the response.
        """
        parents = None
        published_content = None

        def last_content():
            content = published_content
            if content is None or isinstance(content, types.FunctionType):
                return parents[0] if parents is not None else None
            return content

        try:
            self.start()

            # First check for "cancel" redirect ZMI-o-hardcoded thing:
            submit = self.request.get('SUBMIT', None)
            if submit is not None:
                if submit.strip().lower() == 'cancel':
                    cancel = self.request.get('CANCEL_ACTION','')
                    if cancel:
                        raise zExceptions.Redirect(cancel)

            path = self.request.get('PATH_INFO')
            self.request['PARENTS'] = parents = [self.app.application,]

            # Get the virtual host story running
            # This should be in request __init__ but it needs
            # self.request['PARENTS'] to be set properly.
            if 'HTTP_X_VHM_HOST' in self.request.environ:
                set_virtual_host(
                    self.request,
                    self.request.environ['HTTP_X_VHM_HOST'])

            # Get object to publish/render
            published_content = self.request.traverse(
                path, validated_hook=Zope2.zpublisher_validated_hook)
            __traceback_supplement__ = (ErrorSupplement, published_content)

            notify(PubAfterTraversal(self.request))

            # Render the object into the response
            self.app.transaction.recordMetaData(published_content, self.request)
            result = mapply(published_content, self.request.args, self.request,
                            call_object, 1,
                            missing_name, dont_publish_class,
                            self.request, bind=1)

            if result is not None:
                self.response.setBody(result)
        except (ConflictError, Retry, AbortPublication):
            # Conflict are managed at an higher level
            raise
        except zExceptions.Unauthorized:
            # Manage unauthorized
            log_last_error(self.request, self.response, last_content())
            self.response.setStatus(401)
            try:
                # To be compatible with PAS
                self.response._unauthorized()
            except Exception:
                # The _unauthorised handler using PAS failed
                log_last_error(
                    self.request, self.response, obj=last_content(),
                    extra="Error while processing the unauthorized PAS handler")
                self.response.setStatus(401)
                self.response.setBody("")
                self.response.setHeader(
                    'WWW-Authenticate', 'basic realm="%s"' % self.response.realm)
        except zExceptions.Redirect as error:
            # Redirect
            self.response.redirect(str(error))
        except Exception as error:
            content = last_content()
            log_last_error(self.request, self.response, content)
            if self.response.debug_mode:
                # If debug mode is on, don't render anything for the error.
                raise

            self.error(error, content)

        # Return the result of the response
        return self.result()

    def publish_and_retry(self):
        """Publish the request into the response and retry if it
        fails.
        """
        try:
            data = self.publish()
        except (ConflictError, Retry) as error:
            self.abort()
            self.publication_done = True
            if self.request.supports_retry() and not self.data_sent:
                # If can still retry, and didn't send any data yet, do it.
                logger.info('Conflict, retrying request %s' % (
                        self.request['URL']))
                new_request = self.request.retry()
                try:
                    new_publication = self.__class__(
                        self.app, new_request, self.response)
                    data = new_publication.publish_and_retry()
                    self.publication_done = new_publication.publication_done
                finally:
                    new_request.close()
            else:
                # Otherwise, just render a plain error.
                logger.error('Conflict error for request %s' % (
                        self.request['URL']))
                self.response.setStatus(503)
                self.response.setBody(RETRY_FAIL_ERROR_TEMPLATE)
                data = self.result()
        return data

    def __call__(self):
        """Publish the request and send the result via an iterator.
        """
        try:
            data = self.publish_and_retry()
            self.response.startWSGIResponse()
            return WSGIResult(self.request, self, data)
        except Exception:
            # In case of exception we didn't catch (like
            # AbortPublication), abort all the current transaction.
            self.abort()
            self.request.close()
            raise


class WSGIApplication(object):
    """Zope WSGI application.
    """

    def __init__(self, application, transaction, default_handle_errors=True):
        self.application = application
        self.transaction = transaction
        self.memory_maxsize = 2 << 20
        self.default_handle_errors = default_handle_errors

    def save_input(self, environ):
        """We want to save the request input in order to be able to
        retry a request: Zope need to be able to do .seek(0) on
        wsgi_input.
        """
        original_input = environ.get('wsgi.input')

        if original_input is not None:
            length = int(environ.get('CONTENT_LENGTH', '0'))
            if length > self.memory_maxsize:
                new_input = environ['wsgi.input'] = TemporaryFile('w+b')
            else:
                new_input = environ['wsgi.input'] = StringIO()
            to_read = length

            try:
                while to_read:
                    if to_read <= CHUNK_SIZE:
                        data = original_input.read(to_read)
                        to_read = 0
                    else:
                        data = original_input.read(CHUNK_SIZE)
                        to_read -= CHUNK_SIZE
                    new_input.write(data)
            except (socket.error, IOError):
                raise AbortPublication(started=False)

            new_input.seek(0)
            environ['wsgi.input'] = new_input

    def __call__(self, environ, start_response):
        """WSGI entry point.
        """
        try:
            debug_mode = not environ.get(
                'wsgi.handleErrors', self.default_handle_errors)
            self.save_input(environ)
            response = WSGIResponse(environ, start_response, debug_mode)
            request = WSGIRequest(environ['wsgi.input'], environ, response)
            publication = WSGIPublication(self, request, response)

            return publication()
        except AbortPublication, error:
            if not error.response_started:
                msg = 'Socket error'
                start_response('400 Bad Request', [
                        ('Content-Type', 'text/plain'),
                        ('Content-Length', str(len(msg))),
                        ])
                return [msg]
            # Return nothing otherwise
            return []



ERROR_WHILE_RENDERING_ERROR_TEMPLATE = u"""
<html>
  <head>
    <title>Error</title>
  </head>
  <body>
     <h1>An error happened while rendering an error message</h1>
     <p>Sorry for the inconvenience.</p>
     <p>Please check the server logs for more information.</p>
  </body>
</html>
"""

DEFAULT_ERROR_TEMPLATE = u"""
<html>
  <head>
    <title>Error</title>
  </head>
  <body>
     <h1>An error happened</h1>
     <p>Sorry for the inconvenience.</p>
     <p>Please check the server logs for more information.</p>
  </body>
</html>
"""

RETRY_FAIL_ERROR_TEMPLATE = u"""
<html>
  <head>
    <title>Service temporarily unavailable</title>
  </head>
  <body>
     <h1>Your request cannot be processed at the moment</h1>
     <p>Please retry later.</p>
  </body>
</html>
"""





# Copyright (c) 2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from five import grok
from zope.interface import providedBy
from zope.publisher.interfaces import INotFound
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.security.interfaces import IForbidden

from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
import Acquisition

grok.layer(IBrowserRequest)

HTML_TEMPLATE = u"""
<html>
  <head>
    <title>%s</title>
  </head>
  <body>
     <h1>An error happened</h1>
     <p><b>%s</b></p>
  </body>
</html>
"""

class DefaultError(Acquisition.Implicit):
    """Wrapper for errors. The are the context, with all acquisition
    and Zope Component Architecture working.
    """
    security = ClassSecurityInfo()
    security.declareObjectPublic()

    def __init__(self, error):
        self.error = error

    @property
    def __provides__(self):
        if hasattr(self.error, '__provides__'):
            return self.error.__provides__
        return providedBy(self.error)


InitializeClass(DefaultError)


class NotFound(grok.View):
    grok.name('error.html')
    grok.context(INotFound)

    def update(self):
        self.response.setStatus(404)

    def render(self):
        return HTML_TEMPLATE % (
            self.__class__.__name__, 'Page not found: %s' %
            str(self.context.error))


class Forbidden(grok.View):
    grok.name('error.html')
    grok.context(IForbidden)

    def update(self):
        self.response.setStatus(403)

    def render(self):
        return HTML_TEMPLATE % (
            self.__class__.__name__, str(self.context.error))


class Error(grok.View):
    grok.name('error.html')
    grok.context(Exception)

    def update(self):
        self.response.setStatus(500)

    def render(self):
        return HTML_TEMPLATE % (
            self.context.error.__class__.__name__, str(self.context.error))



# Copyright (c) 2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

import unittest

from zope.interface import implements
from zope.publisher.interfaces.http import IResult
from zope.event import notify

from ZODB.POSException import ConflictError
from ZPublisher.Iterators import IStreamIterator
import zExceptions

import infrae.wsgi
from infrae.testing import ZCMLLayer, get_event_names
from infrae.wsgi.publisher import WSGIPublication
from infrae.wsgi.response import WSGIResponse
from infrae.wsgi.tests.mockers import (
    MockWSGIStartResponse, MockTransactionManager, MockRequest, MockApplication)


# Some test views

def hello_view():
    return 'Hello world!'


def no_content_view():
    return u''


def bugous_view():
    raise ValueError("I am not happy")


def not_found_view():
    raise zExceptions.NotFound("I am not here!")


def redirect_view():
    raise zExceptions.Redirect("http://infrae.com/products/silva")


def unauthorized_view():
    raise zExceptions.Unauthorized("Please authenticate")


def forbidden_view():
    raise zExceptions.Forbidden("Go away")


# I don't know how to track this in a better way than a global variable.
confict_count = 0

def not_so_conflictuous_view():
    global conflict_count
    if conflict_count != 0:
        conflict_count -= 1
        raise ConflictError()
    return u'I worked fine'


class TestNextCalled(object):
    """Event triggered each time next() of a TestResult is called.
    """


class TestResult(object):
    """An IResult iterator returning data.
    """
    implements(IResult)

    def __init__(self, data, fail=False):
        self.__next = iter(data).next
        self.__fail = fail

    def __iter__(self):
        return self

    def next(self):
        notify(TestNextCalled())
        # Error testing
        if self.__fail:
            raise ValueError()
        # Else return data
        return self.__next()


def result_view():
    return TestResult(['Hello ', 'World'])


def bugous_result_view():
    return TestResult(['Hello ', 'World'], fail=True)


class TestStreamIterator(object):
    """An IStreamIterator iterator returning data.
    """
    implements(IStreamIterator)

    def __init__(self, data, length, fail=False):
        self.__next = iter(data).next
        self.__length = length
        self.__fail = fail

    def next(self):
        notify(TestNextCalled())
        # Error testing
        if self.__fail:
            raise ValueError()
        return self.__next()

    def __len__(self):
        return self.__length


def streamiterator_view():
    return TestStreamIterator(['It\'s the ', 'world.'], 42)


def bugous_streamiterator_view():
    return TestStreamIterator(['It\'s the ', 'world.'], 42, fail=True)


def consume_wsgi_result(iterator):
    result = ''
    try:
        for piece in iterator:
            result += str(piece)
    finally:
        if hasattr(iterator, 'close'):
            iterator.close()
    return result


class PublisherTestCase(unittest.TestCase):
    """Test that the publisher triggers the correct actions at the
    correct time with the help of mockers.
    """
    layer = ZCMLLayer(infrae.wsgi)

    def setUp(self):
        class WSGIApplication(object):
            transaction = MockTransactionManager()
            application = MockApplication()
            response = MockWSGIStartResponse()

        self.app = WSGIApplication()
        self.response = WSGIResponse({}, self.app.response)

    def new_request_for(self, method):
        # Help to create a request that will be rendered by the given view.
        return MockRequest(
            data={'PATH_INFO': '/', 'URL': 'http://infrae.com'},
            view=method,
            response=self.response,
            retry=2)

    def test_hello_view(self):
        """Test a working view which says hello world.
        """
        request = self.new_request_for(hello_view)
        publication = WSGIPublication(self.app, request, self.response)
        result = publication()

        self.assertEqual(
            request.mocker_called(),
            [('processInputs', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(),
            [('begin', (), {}),
             ('recordMetaData', (hello_view, request), {}),
             ('commit', (), {})])
        self.assertEqual(
            self.app.response.status, '200 OK')
        self.assertEqual(
            self.app.response.headers,
            [('Content-Length', '12'),
             ('Content-Type', 'text/html;charset=utf-8')])
        self.assertEqual(
            get_event_names(),
            ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])

        body = consume_wsgi_result(result)

        self.assertEqual(body, 'Hello world!')
        self.assertEqual(
            request.mocker_called(),  [('close', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(), [])
        self.assertEqual(
            get_event_names(), [])

    def test_no_content(self):
        """Test a working view with no content.
        """
        request = self.new_request_for(no_content_view)
        publication = WSGIPublication(self.app, request, self.response)
        result = publication()

        self.assertEqual(
            request.mocker_called(),
            [('processInputs', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(),
            [('begin', (), {}),
             ('recordMetaData', (no_content_view, request), {}),
             ('commit', (), {})])
        self.assertEqual(
            self.app.response.status, '204 No Content')
        self.assertEqual(
            self.app.response.headers,
            [('Content-Length', '0')])
        self.assertEqual(
            get_event_names(),
            ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])

        body = consume_wsgi_result(result)

        self.assertEqual(body, '')
        self.assertEqual(
            request.mocker_called(),  [('close', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(), [])
        self.assertEqual(
            get_event_names(), [])

    def test_result(self):
        """Test a view that return an object of type IResult. Since it
        is an iterator, commit/close will be only called after the
        iterator has be consumed.
        """
        request = self.new_request_for(result_view)
        publication = WSGIPublication(self.app, request, self.response)
        result = publication()

        self.assertEqual(
            request.mocker_called(),
            [('processInputs', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(),
            [('begin', (), {}),
             ('recordMetaData', (result_view, request), {})])
        self.assertEqual(
            self.app.response.status, '200 OK')
        self.assertEqual(
            self.app.response.headers,
            [('Content-Type', 'text/html;charset=utf-8')])
        self.assertEqual(
            get_event_names(),
            ['PubStart', 'PubAfterTraversal', 'PubBeforeStreaming'])

        body = consume_wsgi_result(result)

        self.assertEqual('Hello World', body)
        self.assertEqual(
            request.mocker_called(),  [('close', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(), [('commit', (), {})])
        self.assertEqual(
            get_event_names(),
            ['TestNextCalled', 'TestNextCalled', 'TestNextCalled',
             'PubBeforeCommit', 'PubSuccess'])

    def test_bugous_result(self):
        """test a view that return an IResult object, and does an
        error. The error is reported through the WSGI stack, but the
        transaction is aborted and the request is closed.
        """
        request = self.new_request_for(bugous_result_view)
        publication = WSGIPublication(self.app, request, self.response)
        result = publication()

        self.assertEqual(
            request.mocker_called(),
            [('processInputs', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(),
            [('begin', (), {}),
             ('recordMetaData', (bugous_result_view, request), {})])
        self.assertEqual(
            self.app.response.status, '200 OK')
        self.assertEqual(
            self.app.response.headers,
            [('Content-Type', 'text/html;charset=utf-8')])
        self.assertEqual(
            get_event_names(),
            ['PubStart', 'PubAfterTraversal', 'PubBeforeStreaming'])

        self.assertRaises(ValueError, consume_wsgi_result, result)

        self.assertEqual(
            request.mocker_called(),  [('close', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(), [('abort', (), {})])
        self.assertEqual(
            get_event_names(),
            ['TestNextCalled', 'PubBeforeAbort', 'PubFailure'])

    def test_result_with_conflict_error(self):
        """Test a view that return an object of type IResult, and does
        conflict error. Like for other iterator errors, the error is
        going throught the middleware stack, but the transaction is
        abort and the request is closed.
        """
        self.app.transaction.mocker_set_conflict(True)

        request = self.new_request_for(result_view)
        publication = WSGIPublication(self.app, request, self.response)
        result = publication()

        self.assertEqual(
            request.mocker_called(),
            [('processInputs', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(),
            [('begin', (), {}),
             ('recordMetaData', (result_view, request), {})])
        self.assertEqual(
            self.app.response.status, '200 OK')
        self.assertEqual(
            self.app.response.headers,
            [('Content-Type', 'text/html;charset=utf-8')])
        self.assertEqual(
            get_event_names(),
            ['PubStart', 'PubAfterTraversal', 'PubBeforeStreaming'])

        self.assertRaises(ConflictError, consume_wsgi_result, result)

        self.assertEqual(
            request.mocker_called(),  [('close', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(),
            [('commit', (), {}),  ('abort', (), {})])
        self.assertEqual(
            get_event_names(),
            ['TestNextCalled', 'TestNextCalled', 'TestNextCalled',
             'PubBeforeCommit', 'PubBeforeAbort', 'PubFailure'])

    def test_streamiterator(self):
        """Test a view that return an object of type
        IStreamIterator. It is basically an iterator like IResult, but
        it is able to give its size. commit/close will be called only
        at the end of the iteration.
        """
        request = self.new_request_for(streamiterator_view)
        publication = WSGIPublication(self.app, request, self.response)
        result = publication()

        self.assertEqual(
            request.mocker_called(),
            [('processInputs', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(),
            [('begin', (), {}),
             ('recordMetaData', (streamiterator_view, request), {})])
        self.assertEqual(
            self.app.response.status, '200 OK')
        self.assertEqual(
            self.app.response.headers,
            [('Content-Length', '42'),
             ('Content-Type', 'text/html;charset=utf-8')])
        self.assertEqual(
            get_event_names(),
            ['PubStart', 'PubAfterTraversal'])

        body = consume_wsgi_result(result)

        self.assertEqual('It\'s the world.', body)
        self.assertEqual(
            request.mocker_called(),  [('close', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(), [('commit', (), {})])
        self.assertEqual(
            get_event_names(),
            ['TestNextCalled', 'TestNextCalled', 'TestNextCalled',
             'PubBeforeCommit', 'PubSuccess'])

    def test_bugous_streamiterator(self):
        """Test a view that return an object of type IStreamIterator,
        and does an error. The error is raised through the WSGI stack,
        but the transaction is aborted and the request is closed.
        """
        request = self.new_request_for(bugous_streamiterator_view)
        publication = WSGIPublication(self.app, request, self.response)
        result = publication()

        self.assertEqual(
            request.mocker_called(),
            [('processInputs', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(),
            [('begin', (), {}),
             ('recordMetaData', (bugous_streamiterator_view, request), {})])
        self.assertEqual(
            self.app.response.status, '200 OK')
        self.assertEqual(
            self.app.response.headers,
            [('Content-Length', '42'),
             ('Content-Type', 'text/html;charset=utf-8')])
        self.assertEqual(
            get_event_names(),
            ['PubStart', 'PubAfterTraversal'])

        self.assertRaises(ValueError, consume_wsgi_result, result)

        self.assertEqual(
            request.mocker_called(),  [('close', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(), [('abort', (), {})])
        self.assertEqual(
            get_event_names(),
            ['TestNextCalled', 'PubBeforeAbort', 'PubFailure'])

    def test_streamiterator_with_conflict_error(self):
        """Test a view that return an object of type IStreamIterator,
        but does a conflict error while committing. The error goes
        throught the WSGI stack, but the transaction is aborted and
        the request is closed.
        """
        self.app.transaction.mocker_set_conflict(True)

        request = self.new_request_for(streamiterator_view)
        publication = WSGIPublication(self.app, request, self.response)
        result = publication()

        self.assertEqual(
            request.mocker_called(),
            [('processInputs', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(),
            [('begin', (), {}),
             ('recordMetaData', (streamiterator_view, request), {})])
        self.assertEqual(
            self.app.response.status, '200 OK')
        self.assertEqual(
            self.app.response.headers,
            [('Content-Length', '42'),
             ('Content-Type', 'text/html;charset=utf-8')])
        self.assertEqual(
            get_event_names(),
            ['PubStart', 'PubAfterTraversal'])

        self.assertRaises(ConflictError, consume_wsgi_result, result)

        self.assertEqual(
            request.mocker_called(),  [('close', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(),
            [('commit', (), {}), ('abort', (), {})])
        self.assertEqual(
            get_event_names(),
            ['TestNextCalled', 'TestNextCalled', 'TestNextCalled',
             'PubBeforeCommit', 'PubBeforeAbort', 'PubFailure'])

    def test_bugous_view(self):
        """Test a broken view.
        """
        request = self.new_request_for(bugous_view)
        publication = WSGIPublication(self.app, request, self.response)
        result = publication()

        self.assertEqual(
            request.mocker_called(),
            [('processInputs', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(),
            [('begin', (), {}),
             ('recordMetaData', (bugous_view, request), {}),
             ('abort', (), {})])
        self.assertEqual(
            self.app.response.status, '500 Internal Server Error')
        self.assertEqual(
           self.app.response.headers,
           [('Content-Length', '150'),
            ('Content-Type', 'text/html;charset=utf-8')])
        self.assertEqual(
            get_event_names(),
            ['PubStart', 'PubAfterTraversal', 'PubBeforeAbort', 'PubFailure'])

        body = consume_wsgi_result(result)

        self.failUnless('I am not happy' in body)
        self.assertEqual(
            request.mocker_called(),  [('close', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(), [])
        self.assertEqual(
            get_event_names(), [])

    def test_not_found(self):
        """Test a view which does a not found exception.
        """
        request = self.new_request_for(not_found_view)
        publication = WSGIPublication(self.app, request, self.response)
        result = publication()

        self.assertEqual(
            request.mocker_called(),
            [('processInputs', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(),
            [('begin', (), {}),
             ('recordMetaData', (not_found_view, request), {}),
             ('abort', (), {})])
        self.assertEqual(self.app.response.status, '404 Not Found')
        self.assertEqual(
            self.app.response.headers,
            [('Content-Length', '164'),
             ('Content-Type', 'text/html;charset=utf-8')])
        self.assertEqual(
            get_event_names(),
            ['PubStart', 'PubAfterTraversal', 'PubBeforeAbort', 'PubFailure'])

        body = consume_wsgi_result(result)

        self.failUnless('Page not found' in body)
        self.assertEqual(
            request.mocker_called(),  [('close', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(), [])
        self.assertEqual(
            get_event_names(), [])

    def test_redirect(self):
        """Test a view which raise a Redirect exception.
        """
        request = self.new_request_for(redirect_view)
        publication = WSGIPublication(self.app, request, self.response)
        result = publication()

        self.assertEqual(
            request.mocker_called(),
            [('processInputs', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(),
            [('begin', (), {}),
             ('recordMetaData', (redirect_view, request), {}),
             ('commit', (), {})])
        self.assertEqual(self.app.response.status, '302 Moved Temporarily')
        self.assertEqual(
            self.app.response.headers,
            [('Content-Length', '0'),
             ('Location', 'http://infrae.com/products/silva')])
        self.assertEqual(
            get_event_names(),
            ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])

        body = consume_wsgi_result(result)

        self.assertEqual(body, '')
        self.assertEqual(
            request.mocker_called(),  [('close', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(), [])
        self.assertEqual(
            get_event_names(), [])

    def test_unauthorized(self):
        """Test a view which raises an Unauthorized exception.
        """
        request = self.new_request_for(unauthorized_view)
        publication = WSGIPublication(self.app, request, self.response)
        result = publication()

        self.assertEqual(
            request.mocker_called(),
            [('processInputs', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(),
            [('begin', (), {}),
             ('recordMetaData', (unauthorized_view, request), {}),
             ('commit', (), {})])
        self.assertEqual(self.app.response.status, '401 Unauthorized')
        self.assertEqual(
            self.app.response.headers,
            [('Content-Length', '0'),
             ('Www-Authenticate', 'basic realm="Zope"')])
        self.assertEqual(
            get_event_names(),
            ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])

        body = consume_wsgi_result(result)

        self.assertEqual(body, '')
        self.assertEqual(
            request.mocker_called(),  [('close', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(), [])
        self.assertEqual(
            get_event_names(), [])

    def test_bugous_pas_handler(self):
        """Test than an error in a PAS unauthorized handler is contained.
        """
        request = self.new_request_for(unauthorized_view)
        response = WSGIResponse({}, self.app.response)
        response._unauthorized = bugous_view # Set the bugous handler
        publication = WSGIPublication(self.app, request, response)
        result = publication()
        self.assertEqual(
            request.mocker_called(),
            [('processInputs', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(),
            [('begin', (), {}),
             ('recordMetaData', (unauthorized_view, request), {}),
             ('commit', (), {})])
        self.assertEqual(self.app.response.status, '401 Unauthorized')
        self.assertEqual(
            self.app.response.headers,
            [('Content-Length', '0'),
             ('Www-Authenticate', 'basic realm="Zope"')])
        self.assertEqual(
            get_event_names(),
            ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])

        body = consume_wsgi_result(result)

        self.assertEqual(body, '')
        self.assertEqual(
            request.mocker_called(),  [('close', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(), [])
        self.assertEqual(
            get_event_names(), [])

    def test_forbidden(self):
        """Test a view which raises the Forbidden exception.
        """
        request = self.new_request_for(forbidden_view)
        publication = WSGIPublication(self.app, request, self.response)
        result = publication()

        self.assertEqual(
            request.mocker_called(),
            [('processInputs', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(),
            [('begin', (), {}),
             ('recordMetaData', (forbidden_view, request), {}),
             ('commit', (), {})])
        self.assertEqual(self.app.response.status, '403 Forbidden')
        self.assertEqual(
            self.app.response.headers,
            [('Content-Length', '142'),
             ('Content-Type', 'text/html;charset=utf-8')])
        self.assertEqual(
            get_event_names(),
            ['PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])

        body = consume_wsgi_result(result)

        self.failUnless('Go away' in body)
        self.assertEqual(
            request.mocker_called(),  [('close', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(), [])
        self.assertEqual(
            get_event_names(), [])

    def test_conflict_errors(self):
        """Test continuous conflict errors on a regular view.
        """
        self.app.transaction.mocker_set_conflict(True)

        request = self.new_request_for(hello_view)
        publication = WSGIPublication(self.app, request, self.response)
        body = consume_wsgi_result(publication())

        self.assertEqual(
            self.app.transaction.mocker_called(),
            [('begin', (), {}),
             ('recordMetaData', (hello_view, request), {}),
             ('commit', (), {}),
             ('abort', (), {}),
             ('begin', (), {}),
             ('recordMetaData', (hello_view, request), {}),
             ('commit', (), {}),
             ('abort', (), {}),
             ('begin', (), {}),
             ('recordMetaData', (hello_view, request), {}),
             ('commit', (), {}),
             ('abort', (), {})])
        self.assertEqual(self.app.response.status, '503 Service Unavailable')
        self.assertEqual(
            self.app.response.headers,
            [('Content-Length', '198'),
             ('Content-Type', 'text/html;charset=utf-8')])
        self.failUnless('Service temporarily unavailable' in body)
        self.assertEqual(
            get_event_names(),
            ['PubStart', 'PubAfterTraversal',
             'PubBeforeCommit', 'PubBeforeAbort', 'PubFailure',
             'PubStart', 'PubAfterTraversal',
             'PubBeforeCommit', 'PubBeforeAbort', 'PubFailure',
             'PubStart', 'PubAfterTraversal',
             'PubBeforeCommit', 'PubBeforeAbort', 'PubFailure',])

    def test_conflict_error_but_ok(self):
        """Test a view which works after triggering one conflict errors, itself.
        """
        global conflict_count
        conflict_count = 1

        request = self.new_request_for(not_so_conflictuous_view)
        publication = WSGIPublication(self.app, request, self.response)
        body = consume_wsgi_result(publication())

        self.assertEqual(
            request.mocker_called(),
            [('processInputs', (), {}), ('retry', (), {}),
             ('processInputs', (), {}), ('close', (), {}), ('close', (), {})])
        self.assertEqual(
            self.app.transaction.mocker_called(),
            [('begin', (), {}),
             ('recordMetaData', (not_so_conflictuous_view, request), {}),
             ('abort', (), {}),
             ('begin', (), {}),
             ('recordMetaData', (not_so_conflictuous_view, request), {}),
             ('commit', (), {})])
        self.assertEqual(conflict_count, 0)
        self.assertEqual(self.app.response.status, '200 OK')
        self.assertEqual(
            self.app.response.headers,
            [('Content-Length', '13'),
             ('Content-Type', 'text/html;charset=utf-8')])
        self.assertEqual(body, 'I worked fine')
        self.assertEqual(
            get_event_names(),
             ['PubStart', 'PubAfterTraversal', 'PubBeforeAbort', 'PubFailure',
              'PubStart', 'PubAfterTraversal', 'PubBeforeCommit', 'PubSuccess'])


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(PublisherTestCase))
    return suite



# Copyright (c) 2010 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from ZPublisher.HTTPResponse import status_reasons
from ZPublisher.Iterators import IStreamIterator
from ZPublisher.pubevents import PubBeforeStreaming
from zope.publisher.interfaces.http import IResult
from zope.event import notify
import zExceptions

from infrae.wsgi.headers import HTTPHeaders

from cgi import escape
from urllib import quote
import socket
import re


HEAD_REGEXP = re.compile('(<head[^>]*>)', re.I)
BASE_REGEXP = re.compile('(<base.*?>)',re.I)


class AbortPublication(Exception):
    """Exception to abort all the publication process.
    """

    def __init__(self, started=True):
        self.response_started = started


class StreamIteratorIterator(object):
    """Make a IStreamIterator a real iterator, because it lack of an
    __iter__ method ...
    """

    def __init__(self, stream):
        assert IStreamIterator.providedBy(stream)
        self.__stream = stream

    def __iter__(self):
        return self.__stream


def format_cookies(cookies):
    """Format cookies as WSGI HTTP headers.
    """
    formatted_cookies = []
    for name, options in cookies.items():
        cookie = '%s="%s"' % (name, quote(options['value']))
        for key, value in options.items():
            key = key.lower()
            if key == 'expires':
                cookie = '%s; Expires=%s' % (cookie, value)
            elif key == 'domain':
                cookie = '%s; Domain=%s' % (cookie, value)
            elif key == 'path':
                cookie = '%s; Path=%s' % (cookie, value)
            elif key == 'max_age':
                cookie = '%s; Max-Age=%s' % (cookie, value)
            elif key == 'comment':
                cookie = '%s; Comment=%s' % (cookie, value)
            elif key == 'secure' and value:
                cookie = '%s; Secure' % cookie
            # Some browsers recognize this cookie attribute
            # and block read/write access via JavaScript
            elif key == 'http_only' and value:
                cookie = '%s; HTTPOnly' % cookie
        formatted_cookies.append(('Set-Cookie', cookie))
    return formatted_cookies


class WSGIResponse(object):
    """A response object using a WSGI connection

    This Response object knows nothing about ZServer, but tries to be
    compatible with the ZPublisher.HTTPResponse.
    """
    default_charset = 'utf-8'
    realm = 'Zope'

    # This is just need by FSDTMLFile. It should be removed when no
    # DTML files a re needed anymore
    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, environ, start_response, debug_mode=False):
        self.headers = HTTPHeaders()
        self.status = 200
        self.cookies = {}
        self.body = None
        self.debug_mode = debug_mode
        self.__base = None
        self.__environ = environ
        self.__start_response = start_response
        self.__started = False
        self.__write = None

    def redirect(self, location, status=302):
        self.status = status
        self.headers['Location'] = location

    def write(self, data):
        # This is deprecated, please return an iterable instead of
        # using write()
        if self.__write is None:
            self.__write = self.startWSGIResponse(stream=True)
        try:
            if isinstance(data, unicode):
                data = data.encode(self.default_charset)
            self.__write(data)
        except (socket.error, IOError):
            # If we can't write anymore to the socket, abort all the
            # publication process.
            raise AbortPublication(started=True)

    def setBase(self, base):
        # HTTPResponse compatibility: need to randomly insert a base
        # header in HTML.
        if base and not base.endswith('/'):
            base += '/'
        self.__base = base

    def __insertBase(self, body):
        # HTTPResponse compatibility: insert a base tag if
        # needed. There is no way to get ride of that, several ZMI
        # screens rely on this (form using :method variables).
        content_type = self.headers.get('content-type', '').split(';')[0]
        if content_type and (content_type != 'text/html'):
            return body

        if self.__base:
            if body:
                match = HEAD_REGEXP.search(body)
                if match is not None:
                    index = match.start(0) + len(match.group(0))
                    ibase = BASE_REGEXP.search(body)
                    if ibase is None:
                        body = '%s\n<base href="%s" />\n%s' % (
                            body[:index], escape(self.__base, 1), body[index:])
        return body

    def setBody(self, body, **options):
        # We ignore options
        if isinstance(body, basestring):
            body = self.__insertBase(body)
            if isinstance(body, unicode):
                body = body.encode(self.default_charset)
        self.body = body

    def setStatus(self, status, msg=None):
        # We ignore msg and use our own.
        self.status = status

    def getStatus(self):
        return self.status

    def setHeader(self, name, value, literal=0):
        # literal=0 is for HTTPResponse compatibility
        self.headers[name] = value

    addHeader = setHeader

    def getHeader(self, name, literal=0):
        # literal=0 is for HTTPResponse compatibility
        return self.headers.get(name)

    def appendCookie(self, name, value):
        name = str(name)
        value = str(value)

        cookies = self.cookies
        if cookies.has_key(name):
            cookie = cookies[name]
        else:
            cookie = cookies[name] = {}
        if cookie.has_key('value'):
            cookie['value'] = '%s:%s' % (cookie['value'], value)
        else:
            cookie['value'] = value

    def expireCookie(self, name, **options):
        options['max_age'] = 0
        options['expires'] = 'Wed, 31-Dec-97 23:59:59 GMT'
        self.setCookie(name, 'deleted', **options)

    def setCookie(self, name, value, **options):
        name = str(name)
        value = str(value)

        cookies = self.cookies
        if cookies.has_key(name):
            cookie = cookies[name]
        else:
            cookie = cookies[name] = {}
        for cookie_key, cookie_value in options.items():
            cookie[cookie_key] = cookie_value
        cookie['value'] = value

    def startWSGIResponse(self, stream=False):
        if self.__started:
            return self.__write
        self.__started = True

        # If the body is an IResult, it is a case of streaming where
        # we don't fix headers.
        stream = stream or IResult.providedBy(self.body)

        if not stream:
            # If we are not streaming, we try to set Content-Length,
            # Content-Type and adapt status if there is no content.
            if not self.headers.has_key('Content-Length'):
                content_length = None
                if (isinstance(self.body, basestring) or
                    IStreamIterator.providedBy(self.body)):
                    content_length = len(self.body)
                elif self.body is None:
                    content_length = 0
                if content_length is not None:
                    self.headers['Content-Length'] = content_length

            if self.headers.has_key('Content-Length'):
                content_length = self.headers['Content-Length']
                if not content_length and self.status == 200:
                    # Set no content status if there is no content
                    self.status = 204

            if not self.headers.has_key('Content-Type'):
                if content_length:
                    # If there is content and no Content-Type, set HTML
                    self.headers['Content-Type'] = 'text/html;charset=%s' % (
                        self.default_charset,)
        else:
            # Fire event before streaming
            notify(PubBeforeStreaming(self))

            # Fix default Content-Type
            if not self.headers.has_key('Content-Type'):
                self.headers['Content-Type'] = 'text/html;charset=%s' % (
                    self.default_charset,)


        formatted_status = "%d %s" % (
            self.status, status_reasons.get(self.status, 'OK'))
        formatted_headers = self.headers.items() + format_cookies(self.cookies)
        return self.__start_response(formatted_status, formatted_headers)

    def getWSGIResponse(self):
        # This return a tuple (data_sent, data_to_send_to_WSGI)
        result = self.body
        if result is not None:
            # If we have an iterator, we say that data have been sent
            # in order not to commit the transaction finish consuming
            # the iterator.
            if IStreamIterator.providedBy(result):
                return (True, StreamIteratorIterator(result))
            elif IResult.providedBy(result):
                return (True, result)
            return (self.__started, [result,])
        return (self.__started, [])

    def _unauthorized(self):
        # Unauthorized is implemented like this to be compliant with
        self.setHeader(
            'WWW-Authenticate', 'basic realm="%s"' % self.realm)

    # ZPublisher define a method for each error on the response
    # object. We need to add them to be compatible

    def unauthorized(self):
        raise zExceptions.Unauthorized("Please authenticate")

    def debugError(self, name):
        raise zExceptions.NotFound(name)

    def forbiddenError(self, name):
        raise zExceptions.Forbidden(name)

    def badRequestError(self, name):
        raise zExceptions.BadRequest(name)

    def notFoundError(self, name):
        raise zExceptions.NotFound(name)

    # Support for ConflictError/Retry. We reuse the same response.

    def retry(self):
        return self
